#!/bin/bash
#
# Process controller script to manage spool jobs defined in a directory,
# containing bash variables (shell script fragment) defining a job.
# See also /etc/spoolprocess.d/.README
# Usage: call from a cron job.
# Exit codes: 
#   0 - OK
# 101 - missing spool jobs definition directory
#
# 2007-03-26 - garancsi.tamas@ionlab.net - kovacs.zoltan@ionlab.net

CAT="/bin/cat"				# cat command call
CHMOD="/bin/chmod"			# chmod command call
CHOWN="/bin/chown"			# chown command call
CUT="/usr/bin/cut"			# cut command call
DATE="/bin/date"			# date command call
FIND="/usr/bin/find"			# find command call
GREP="/bin/grep"		  	# grep command call
PS="/bin/ps"				# ps command call
SU="/bin/su"				# su command call
TOUCH="/bin/touch"			# touch command call
WC="/usr/bin/wc"			# wc command call

LOGFILE="/var/log/spoolprocess.log"	# Logfile pathname
LOGOWNER="root:adm"			# Logfile owner and group
LOGRIGHTS="640"				# Logfile access rights
PROCESS="/usr/local/bin/process.sh"	# Processor script call
SPOOLDEFS="/etc/spoolprocess.d"		# Job defs - a job per file

MSG_DELETED="deleted"
MSG_SPOOLFAIL="Wrong spool definition"
MSG_STARTED="started"

# Getting common settings (if any)
HEADERS="/etc/default/spoolprocess"
. $HEADERS

# Creating logfile if absent
if [ ! -f "$LOGFILE" ]; then 
    $TOUCH $LOGFILE
    $CHOWN $LOGOWNER $LOGFILE
    $CHMOD $LOGRIGHTS $LOGFILE
fi

# Checking mandatory spool definition directory
[ -d "$SPOOLDEFS" ] || exit 101

# Scanning spool definition directory
ls -D1 "$SPOOLDEFS" 2>/dev/null | while read filename
do
    # Handling a spooler
    # Setting defaults
    SPOOLUSER=""
    PROCMAX=1
    SPOOLDIR=""
    PROCESSOR=""
    PROCESSED=""
    # Getting spool definition file
    if [ -f "$SPOOLDEFS"/"$filename" ]; then
	. "$SPOOLDEFS"/"$filename"
	# Checking mandatory components:
	# valid(?) user, existing spooldir, executable processor
	if [ ! -z "$SPOOLUSER" -a -d "$SPOOLDIR" -a -x "$PROCESSOR" ]; then
	    # Enumerating spoolfiles from this spool (younger than a day)
	    $FIND "$SPOOLDIR" -type f -mtime 0 2>/dev/null | while read spoolfile
	    do
		# Checking no. of already running processes
		if [ `$PS aux | $GREP -vi " $GREP " | $GREP "/bin/bash $PROCESS " | $WC -l` -lt $PROCMAX ]; then
		    # Exclude spoolfile when it's now processing
    		    if [ -z "`$PS aux | $GREP -vi " $GREP " | $GREP " $PROCESS $spoolfile "`" ]; then
			# Launch a processor in background
			$SU - $SPOOLUSER -c "$PROCESS \"$spoolfile\" \"$PROCESSOR\" \"$PROCESSED\"" &
			# Write a log entry about this processor
			echo "\"`$DATE '+%Y-%m-%d %H:%M:%S'`\" \"$MSG_STARTED\" \"$SPOOLUSER\" \"$PROCESS\" \"$spoolfile\" \"$PROCESSOR\" \"$PROCESSED\"" >>$LOGFILE
		    fi
		fi
	    done

	else
	    # Write a log entry about failure
	    echo "\"`$DATE '+%Y-%m-%d %H:%M:%S'`\" \"$MSG_SPOOLFAIL\" \""$SPOOLDEFS"/"$filename"\"" >>$LOGFILE
	fi
    fi
done
